%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%2D transient model of rock matrix, coupled with 1D convective fluid flow
%model.
% - Finite difference model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



function [TfracOUT Tflow Tmatrix Tinjection Tproduction t_vector] = EGSdisc(nx,ny,L,H,W,nt,t_y,Tinitial,Tinlet,m,H_f,K,Tcrust,Depth,r1,r2,m_f)

%Geometrical variables
    n = nx*ny; % total number of nodes

    dx = (L/(nx-1))*ones(ny,nx-1); %equidistant

    Y = [0;logspace(-2,log10(H),ny-1)'];
    dy = Y(2:ny,1) - [Y(1:ny-1,1)];%logaritmic time scale
    dy = dy*ones(1,nx);
    dy = flipud(dy);
    [X Y] = meshgrid(0:dx:L,flipud(Y));%

%Time 
    t = t_y*365*24*60*60; %Total time [sec]
     t_vector = logspace(0,log10(t),nt);
     dt = t_vector(1,1:end) - [0,t_vector(1,1:end-1)]; %logaritmic time scale
    %t_vector = linspace(0,t,nt);
    %dt = t/nt; %linear time

%Inital conditions
    T0 = Tinitial*ones(ny,nx); %inital rock matrix temperature

%physical properties Granite
    Egen = 0*ones(ny,nx); %Internatl heat generation from rock [W]
    %k_r = ones(ny,nx)*2.9; %Thermal conductivity granite [W/m K]
    %Temperature dependent, calculated inside transient for loop...
    rho_r = 2800; %density [kg/m3]
    %Cp_r = ones(ny,nx)*950; %Specific heat capacity [J/kg K]
    %Temperature dependent, calculated inside transient for loop...
    
%physical properties Water
    Cp_w = 4250; %specific heat of water [J/kg K]
    rho_w = 950; %Density water [kg/m^3]
    k_w = 0.68; %Thermal conductivity of water [W/mK]
    Pr = 1.5;
    
%flow conditions
    mu = 0.3*10^(-3); %dynamic viscosity [kg/ms]
   
    %m_f   = m/n_f; %flow rate in each fracture per unit width [kg/(s m)]
    u = m_f/(rho_w*H_f); %Flow velocity [m/s]
    Pe = (rho_w*u)/(k_w/dx(1));
    Re = (rho_w*u*H_f)/(mu);

%Heat transfer coefficient
   AA = 543;
   BB = 0.9;
   h = 100000;% AA*u^BB;   %heat transfer coefficient. based on paper by Zhao et al

%Display info%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
disp(text());

% figure(3)%Plot time and Y discretization
% subplot(2,2,1)
% plot(0:nt-1,t_vector/(365*24*60*60))
% xlabel('Time step')
% ylabel('Time elapsed [years]')
% title('Time discretization')
% subplot(2,2,2)
% plot([ny-1:-1:0]',Y(:,1))
% xlabel('Hight step')
% ylabel('Hight [m]')
% title('Hight discretization')
% subplot(2,2,3)
% plot(1:nt,dt)
% xlabel('Time step')
% ylabel('dt [sec]')
% title('dt discretization')
% subplot(2,2,2)
% plot([ny-1:-1:1]',dy(:,1))
% xlabel('Hight step')
% ylabel('Hight [m]')
% title('dy discretization')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Main calculation%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Call nested functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
m
m_f
    
%transient loop%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 for ii = 1:nt 
    dt1 = dt(ii); %specify dt at given timestep, logarithmic
    t0 = t_vector(ii);
    
    [Cp_r] = CP(T0); %Specific heat capacity [J/kg K]
    [k_r] = K_r(T0); %Thermal conductivity granite [W/m K]
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Injection well %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if t0 < 24*60*60 %Ramey's wellbore model unstable for low time values... (first transient periode)
          Tinjection(ii) = Tinlet;
          TinjectionWELL = Tinlet;
     else
          [TinjectionWELL] = injectionwell(Tinlet);
          Tinjection(ii) = TinjectionWELL(end);
     end
     
     Tin = TinjectionWELL(end); %temperature bottom of injection wellbore
     
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     %Fracture and formation calculation%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     
     %Set up coefficient matrix (not outside loop since k_r change each loop...   
     [AE_f AN_f AP_f AW_f AN_r AW_r AP_const_r AE_r AS_r AcX AcY dx1 dy1 Asur] = rockflowcoupleCOEFF();
  
     %Transient solution of fracture
     [Tmatrix Tflow Tcoupled] = transientCOUPLED(T0);
        T0 = Tmatrix;
        TfracOUT(ii) = Tflow(nx); %Temperature bottom of production well (used in production well calculation)
     
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     %Production well   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     if t0 < 24*60*60 %Ramey's wellbore model unstable for low time values... (first transient periode)
          Tproduction(ii) = Tflow(nx);
     else
          [TproductionWELL] = productionwell(Tflow(nx));
          Tproduction(ii) = TproductionWELL(end); %Temperature top site at production well
     end
     
 %Check results, out of bounds?
 if max(Tflow)>Tinitial+1
     disp(['temperture of fluid out of bounds.. with a max temperature of ',num2str(max(Tflow)),'C']);
     disp(['nt = ',num2str(ii),' and timestep = ',num2str(dt1),' at time = ',num2str(t_vector(ii)/(365*24*60*60)),' years']);
          
     return
 end
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%store some values for later use/plots and graphs%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if ii == 1
        TflowSTORED{1} = Tflow;
        TmatrixSTORED{1} = Tmatrix;

        elseif ii == nt*0.5
        TflowSTORED{2} = Tflow;
        TmatrixSTORED{2} = Tmatrix;

        elseif ii == nt*0.8
        TflowSTORED{3} = Tflow;
        TmatrixSTORED{3} = Tmatrix;

        elseif ii == nt*0.9
        TflowSTORED{4} = Tflow;
        TmatrixSTORED{4} = Tmatrix;

        elseif ii == nt
        TflowSTORED{5} = Tflow;
        TmatrixSTORED{5} = Tmatrix;
        end
 
 end 
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Call Analytic functions and PLTOS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Analytic flow between flat plates
     [Tflowconst] = constsurf(L,Tinitial,Tin,h,m_f,Cp_w); 
% Semi inifinite solid
     [Tsemih YY] = semisolid(k_r,rho_r,Cp_r,h,Tinitial,Tin,t,H);
    
% Analytical EGS
      [TflowAN TmatrixAN] = EGSanalytic(k_r,rho_r,rho_w,Cp_r,u,H_f,Cp_w,Tin,Tinitial,X,Y,t,nx,t_vector); %Call analytic EGS function
 
% PLOTS%%
%      PLOTS(TflowAN,TmatrixAN,TmatrixSTORED,Tsemih,Tflowconst,TflowSTORED,X,Y,h,Pe,Re,dx,dy,Tinitial,L,ny,nx,t_y,nt,t_vector,YY,m_f)
 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%NESTED Functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Injection wellbore
   function [Tinjection] = injectionwell(Tinlet)
%WELLBORE calculates wellbore heat transmission based on Ramey's analytical
%wellbore heat transmission model from 1962
b = Tcrust;
t_days = t0/(24*60*60);
k_casing = 64; %AISI 1000
k_rwell = 2.51;

alpha = 8*10^-7; %%Thermal diffusivity of rock based on Kanev et al 1997 


D = [0:10:Depth]'; %depth
a = ones(length(D),1)*K; %Geothermal temperature coefficient gradient


t0dim = alpha*t0./(r2^2);

%parameters
f = - log(r2./(2*sqrt(alpha*t0))) - 0.290;
Re_tube = (2*m)./(mu*pi*r1);

%Base heat transfer coefficient on Reynolds number
if Re_tube > 10000 %turbulent flow
   h1 = (k_w/(2*r1))*0.021*Pr^0.5*Re_tube^0.8;
elseif Re_tube<2300 %laminar
    h1 = 4.364*k_w/(2*r1);
else
    disp('transient wellbore flow.. h for turbulent used.')
       h1 = (k_w/(2*r1))*0.021*Pr^0.5*Re_tube.^0.8;
end

Tgeo = a.*D + b; %Geothermal temperature gradient for injection plot

  U = (1./h1 + (r2-r1)/k_casing).^(-1);
  A = ones(length(D(:,1)),1)*(m.*Cp_w.*(k_rwell + r1*U*f)./(2*pi*k_rwell*U*r1));
  A1 = ones(length(D(:,1)),1)*m*Cp_w.*f./(2*pi*k_rwell);
   

Tinjection = a.*D + b - a.*A + (Tinlet + a.*A - b).*exp(-D./A);




% figure(13) %Plot of temperature profile
% D1 = -D(:,1);
% plot(Tgeo(:,1),D1(:,1),'--',Tinjection,D1)
% ylabel('Depth [m]')
% xlabel('Temperature [C]')
% title({'Injection Wellbore temperature as a function of time and depth';['Injection temperature = ',num2str(Tinlet),'C - inner radius of wellbore = ',num2str(r1),'m'];['After ',num2str(t_days),' days']})
% legend({'Geothermal temperature profile', ['Wellbore temperature m =',num2str(m),'kg/s']})
   end

%Rock matrix calculation
    %Rock matrix coefficient matrix
    function [AE_f AN_f AP_f AW_f AN_r AW_r AP_const_r AE_r AS_r AcX AcY dx1 dy1 Asur] = rockflowcoupleCOEFF()
    AcX = [W*dx(:,1)/2,W*dx];AcX(:,nx) = AcX(:,nx)/2 ;%Cross sectional area in y-direction
    AcY = ([zeros(1,nx);dy]+[dy;zeros(1,nx)])/2 ;%Cross sectional area in x-direction, adjusted for discretizied dy

    dx1 = [dx,zeros(ny,1)]; %physical correct representation of dx, LAST coloumn outside grid (set = 0)
    dy1 = [dy;zeros(1,nx)]; %physical correct representation of dy, LAST row outside grid (set = 0)
   
    %FLUID FLOW COEFFICIENTS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Asur = AcX(ny,:); %area per unit width
    
    %A matrix - coefficient matrix
    %interior nodes

    awI = k_r(2:ny-1,2:nx-1).*AcY(2:ny-1,2:nx-1)./dx1(2:ny-1,1:nx-2);
    aeI = k_r(2:ny-1,2:nx-1).*AcY(2:ny-1,2:nx-1)./dx1(2:ny-1,1:nx-2);
    asI = k_r(2:ny-1,2:nx-1).*AcX(2:ny-1,2:nx-1)./dy1(2:ny-1,2:nx-1);
    anI = k_r(2:ny-1,2:nx-1).*AcX(2:ny-1,2:nx-1)./dy1(1:ny-2,2:nx-1);
    
    apI = awI + aeI + asI + anI; 
    
    %Exterior nodes
    %North surface q = 0
    awN = k_r(1,2:nx-1).*AcY(1,2:nx-1)./dx1(1,2:nx-1);
    aeN = k_r(1,2:nx-1).*AcY(1,2:nx-1)./dx1(1,2:nx-1);
    anN = zeros(1,nx-2);
    asN = k_r(1,2:nx-1).*AcX(1,2:nx-1)./dy1(1,2:nx-1);
   
    apN = awN + aeN + asN;
    
    %West surface q = 0
    awW = zeros(ny-2,1);
    aeW = k_r(2:ny-1,1).*AcY(2:ny-1,1)./dx1(2:ny-1,1);
    anW = k_r(2:ny-1,1).*AcX(2:ny-1,1)./dy1(1:ny-2,1);
    asW = k_r(2:ny-1,1).*AcX(2:ny-1,1)./dy1(2:ny-1,1);
   
    apW = aeW + anW + asW; 
    
    %East surface q = 0
    awE = k_r(2:ny-1,nx).*AcY(2:ny-1,nx)./dx1(2:ny-1,nx-1);
    aeE = zeros(ny-2,1);
    anE = k_r(2:ny-1,nx).*AcX(2:ny-1,nx)./dy1(1:ny-2,nx);
    asE = k_r(2:ny-1,nx).*AcX(2:ny-1,nx)./dy1(2:ny-1,nx);
    
    apE = awE + anE + asE;
    
    %South surface q = qs
    awS = k_r(ny,2:nx-1).*AcY(ny,2:nx-1)./dx1(ny,2:nx-1);
    aeS = k_r(ny,2:nx-1).*AcY(ny,2:nx-1)./dx1(ny,2:nx-1);
    anS = k_r(ny,2:nx-1).*AcX(ny,2:nx-1)./dy1(ny-1,2:nx-1);
    asS = zeros(1,nx-2);
   
    apS = awS + aeS + anS;
    
    %Corner nodes
    aw11 = 0; an11 = 0; as11 = k_r(1,1).*AcX(1,1)./dy1(1,1); ae11 = k_r(1,1).*AcY(1,1)./dx1(1,1);ap11 = as11+ae11; %top left corner
    ae1nx = 0; an1nx = 0; as1nx = as11; aw1nx = k_r(1,nx).*AcY(1,nx)./dx1(1,nx-1);ap1nx = as1nx + aw1nx;  %top right corner
    awny1 = 0; asny1 = 0; anny1 = k_r(ny,1).*AcX(ny,1)./dy1(ny-1,1); aeny1=k_r(ny,1)*AcY(ny,1)./dx1(ny,1);apny1 = anny1 + aeny1;%bottom left corner
    aenynx = 0; asnynx = 0; annynx = k_r(ny,nx).*AcX(ny,nx)./dy1(ny-1,nx); awnynx=k_r(ny,nx).*AcY(ny,nx)./dx1(ny,nx-1); apnynx = annynx + awnynx;%bottom right corner
    
    %Combine interior, exterior and corner coefficients
    AW_r = [awN,aw1nx,aw11;awI,awE,awW;awS,awnynx,awny1;zeros(1,nx)];
    AE_r = [ae1nx,ae11,aeN;aeE,aeW,aeI;aenynx,aeny1,aeS;zeros(1,nx)];
    AN_r = [anW,anI,anE;anny1,anS,annynx;an11,anN,an1nx;zeros(1,nx)];

    AS_r = [asny1,asS,asnynx;as11,asN,as1nx;asW,asI,asE;Asur*h];%AS_r(ny+1,1) = 0;
    
    AP_const_r = [ap11,apN,ap1nx;apW,apI,apE;apny1,apS,apnynx;zeros(1,nx)];
    AP_const_r(ny,:) = AP_const_r(ny,:) + Asur*h;

  %FLUID FLOW%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %UPFLOW scheme%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %internal nodes
   AP_f = [zeros(ny,nx);m_f*Cp_w/2 + Asur*h + 2*(H_f/2)*k_w./dx(ny,nx-1)];AP_f(ny+1,1)=1;%last part adjust for inlet node
   AW_f = [zeros(ny,nx);ones(1,nx-1)*m_f*Cp_w/2 + (H_f/2)*k_w./dx(ny,nx-1),0];
   AN_f = [zeros(ny-1,nx);Asur*h;zeros(1,nx)];AN_f(ny,1)=0;
   AE_f = [zeros(ny,nx);0,ones(1,nx-1)*(H_f/2)*k_w/dx(ny,nx-1)] ;
    end

    %Transient rock matrix calculation
    function [Tmatrix Tflow Tcoupled] = transientCOUPLED(T0)
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %SET UP COUPLED A MATRIX%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       
        %ROCK MATRIX Transient term%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
               ap0 =rho_r*Cp_r.*AcX.*AcY/dt1;
             
         
               AP_r = [AP_const_r(1:ny,1:nx) + ap0;zeros(1,nx)];
               
         %NODE COEFFICIENTS%%%%
      
         AN = AN_r + AN_f;
         AS = AS_r;
         AW = AW_r + AW_f;
   
         AE = AE_r + AE_f;
      
         AP = AP_r + AP_f;
         
   %Set up A matrix (sparse)%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   A = spdiags([-reshape(AN',n+nx,1),-reshape(AW',n+nx,1),reshape(AP',n+nx,1),-reshape(AE',n+nx,1),-reshape(AS',n+nx,1)],[-nx -1 0 1 nx],n+nx,n+nx);

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %Boundary matrix%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   b1 = [ap0.*T0;zeros(1,nx)];b1(ny+1,1) = Tin; %interior nodes

   b = reshape(b1',n+nx,1);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %solution of system of linear equations
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    Tcoupled =A\b;
         
    Tmatrix = reshape(Tcoupled(1:n),nx,ny)';
    Tflow = Tcoupled(n+1:n+nx)';
   
end

%Production wellbore
    function [Tproduction] = productionwell(Tfracout)
%WELLBORE calculates wellbore heat transmission based on Ramey's analytical
%wellbore heat transmission model from 1962

b = Tinitial; %Earth temperature top site [C]

t_days = t0/(24*60*60);
k_casing = 64; %AISI 1000
k_rwell = 2.51;

Cp_rwell = 800;
r2 = 0.15;
r1 = 0.12;



D = [0:1:Depth]'; %depth
a = -[ones(length(D),1)*K]; %Geothermal temperature coefficient gradient


%Thermal diffusivity of rock
alpha = 8*10^-7; %based on Kanev et al 1997 

t0dim = alpha*t0./(r2^2);

%parameters
f = - log(r2./(2*sqrt(alpha*t0))) - 0.290;

Re_tube = (2*m)./(mu*pi*r1);

%Base heat transfer coefficient on Reynolds number
if Re_tube > 10000 %turbulent flow
   h1 = (k_w/(2*r1))*0.021*Pr^0.5*Re_tube^0.8;
elseif Re_tube<2300 %laminar
    h1 = 4.364*k_w/(2*r1);
else
    disp('transient wellbore flow.. h for turbulent used.')
       h1 = (k_w/(2*r1))*0.021*Pr^0.5*Re_tube.^0.8;
end

Tgeo = a.*D + b; %Geothermal temperature gradient for injection plot

  U = (1./h1 + (r2-r1)/k_casing).^(-1);

  A = ones(length(D(:,1)),1)*(m.*Cp_w.*(k_rwell + r1*U*f)./(2*pi*k_rwell*U*r1));
  %A1 = ones(length(D(:,1)),1)*m*Cp_w.*f./(2*pi*k_rwell);
   

Tproduction = a.*D + b - a.*A + (Tfracout + a.*A - b).*exp(-D./A);





% figure(12)
% D1 = D(:,1);
% plot(Tgeo(:,1),D1(:,1),'--',Tproduction,D1)
% ylabel('Depth [m]')
% xlabel('Temperature [C]')
% title({'Production Wellbore temperature as a function of time and depth';['Injection temperature = ',num2str(Tfracout),'C - inner radius of wellbore = ',num2str(r1),'m'];['After ',num2str(t_days),' days']})
% legend({'Geothermal temperature profile', ['Wellbore temperature m =',num2str(m),'kg/s']})
   end


%Print and store variables%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function [text] = text()
text = {
    ['Fluid flow and rock matrix calculation: ']
    ['-----------------------------------------------']
    ['Total time: ',num2str(t_y),' years']
    ['number of timesteps: ',num2str(nt)]
    ['Delta X (dx): ',num2str(dx(1,1)),' m']
    ['Number of nodes in x direction is (equidistant): ',num2str(nx)]
    ['Number of nodes in y direction is: ',num2str(ny)]
    ['Total number of nodes: ',num2str(n)]
    ['-----------------------------------------------']
    ['Length of fracture : ',num2str(L),'m']
    ['Distance between fractures ',num2str(2*H),'m']
    ['Hight of fracture: ',num2str(H_f),'m']
 
    ['Total water massflow: ',num2str(m),'kg/s']
    ['Water massflow in one fracture: ',num2str(m_f),'kg/s']
    ['Water velocity in one fracture: ',num2str(u),'m/s']
    ['Reynoldsnumber: ',num2str(Re)]
    ['Peclet number: ',num2str(Pe)]
    ['-----------------------------------------------']
    ['Rock density: ',num2str(rho_r),' kg/m3']
    ['-----------------------------------------------']
    ['Water density: ',num2str(rho_w),'kg/m3']
    ['Water specific heat: ',num2str(Cp_w),' J/kg K']
    ['Water thermal conductivity: ',num2str(k_w),' W/m K']
    ['Heat transfer coefficient: ',num2str(h),' W/(m2K)']  
    };
end

end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Functions%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%PLOTS
function [] = PLOTS(TflowAN,TmatrixAN,TmatrixSTORED,Tsemih,Tflowconst,TflowSTORED,X,Y,h,Pe,Re,dx,dy,Tinitial,L,ny,nx,t_y,nt,t_vector,YY,m_f)
tt = 365*24*60*60;

% figure(1)%Comparison analytic and finite difference
% subplot(1,2,1)
% plot(Tsemih,YY,TmatrixSTORED{5}(:,nx/2),Y(:,1),'x')
% ylabel('distance from surface [m]')
% xlabel('Temperature [C]')
% legend('Semi infinite solid analytic solution','Finite difference profile at x=0')
% title({'Semi infinite solid with constant heat flux';['dx =',num2str(dx(1,1)),'m and dy = ',num2str(dy(1,1)),'m']})
% subplot(1,2,2)
% plot([0:0.1:L],Tflowconst,X(ny,:),TflowSTORED{1}(),'x',X(1,:),TflowSTORED{5},'x',X(1,:),TflowAN)
% ylabel('Temperature [C]')
% xlabel('Distance [m]')
% legend('Analytical solution (INITIAL - flat plates)','Finite difference model - INTIAL (UPWIND scheme)','Finite difference model - FINAL (UPWIND scheme)','Analytic solution Final profile')
% title({'Comparison between analytic model of flow between flat plates and finite difference model';['Surface temperature = ',num2str(Tinitial),'C'];['Peclet number =',num2str(Pe),' heat transfer coefficient =',num2str(h),' and Reynolds number = ',num2str(Re)]})

figure(2)%Temperature at rock matrix surface and vertically
subplot(1,2,1)
plot(X(ny,:),TmatrixSTORED{1}(ny,:),X(ny,:),TmatrixSTORED{2}(ny,:),X(ny,:),TmatrixSTORED{3}(ny,:),X(ny,:),TmatrixSTORED{4}(ny,:),X(ny,:),TmatrixSTORED{5}(ny,:))
xlabel('Distance [m]')
ylabel('Temperature [C]')
legend({'Initial temperature',['Temperature after ', num2str(t_vector(nt*0.5)/tt),' years'],['Temperature after ', num2str(t_vector(nt*0.8)/tt),' years'],['Temperature after ', num2str(t_vector(nt*0.9)/tt),' years'],['Temperature after ', num2str(t_vector(nt)/tt),' years']})
title({'Rock matrix temperature at interface';'Constant specific heat and thermal conductivity of granite';['water mass flow =',num2str(m_f),'kg/s']})
subplot(1,2,2)
plot(TmatrixSTORED{1}(:,nx/2),Y(:,nx/2),TmatrixSTORED{2}(:,nx/2),Y(:,nx/2),TmatrixSTORED{3}(:,nx/2),Y(:,nx/2),TmatrixSTORED{4}(:,nx/2),Y(:,nx/2),TmatrixSTORED{5}(:,nx/2),Y(:,nx/2))
ylabel('Distance [m]')
xlabel('Temperature [C]')
legend({'Initial temperature',['Temperature after ', num2str(t_vector(nt*0.5)/tt),' years'],['Temperature after ', num2str(nt*0.8)/tt,' years'],['Temperature after ', num2str(t_vector(nt*0.9)/tt),' years'],['Temperature after ',num2str(t_vector(nt)/tt),' years']})
title({['Vertical rock matrix temperature profile at ',num2str(L/2),'m'];'Constant specific heat and thermal conductivity of granite';['water mass flow =',num2str(m_f),'kg/s']}) 



figure(4) %final matrix temp
% subplot(1,2,1)
contourf(X,Y,TmatrixSTORED{5},20)
title({'Finite difference model';['Matrix temperatur after ',num2str(t_y),'years'];['timesteps ',num2str(nt),'x']})
xlabel('Length [m]')
ylabel('Height [m]')
colorbar
% subplot(1,2,2)
% contourf(X,Y,TmatrixAN,20)
% title({'Analytical solution';['Matrix temperatur after ',num2str(t_y),'years'];['timesteps ',num2str(nt),'.']})
% xlabel('Length [m]')
% ylabel('Height [m]')
% colorbar

% figure(5) %Fracture temperature 
% plot(X(1,:),TflowSTORED{1}(),X(1,:),TflowSTORED{1}(),X(1,:),TflowSTORED{3}(),X(1,:),TflowSTORED{4}(),X(1,:),TflowSTORED{5}())
% xlabel('Distance [m]')
% ylabel('Temperature [C]')
% legend({'Initial temperature',['Temperature after ', num2str(t_vector(nt*0.5)/tt),' years'],['Temperature after ', num2str(t_vector(nt*0.8)/tt),' years'],['Temperature after ', num2str(t_vector(nt*0.9)/tt),' years'],['Temperature after ',num2str(t_vector(nt)/tt),' years']})
% title(['Fluid flow temperature profile at differente times']) 
end

%Semisolid calculation
function [Tsemih YY] = semisolid(k_r,rho_r,Cp_r,h,Tinitial,Tin,t,H)
   
YY = [H:-0.1:0]';

alpha = k_r(1,1)/(rho_r*Cp_r(1,1));
Tsemih = Tinitial + (Tin - Tinitial)*( erfc(YY/(2*sqrt(alpha*t))) - exp( h*YY/k_r(1,1) + h^2*alpha*t/((k_r(1,1))^2) ).*erfc( YY/(2*sqrt(alpha*t)) + h*sqrt(alpha*t)./k_r(1,1) ) );

end

%constant surface temperature calculation%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Tflowconst] = constsurf(L,Tinitial,Tin,h,m_f,Cp_w)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Analytic solution of convective between flat plates
%   - using heat transfer coefficient and parameters from program
%Following Cengel (Heat and mass transfer: A practical approach)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%input parameters
% Tin = Temperature of surrounding fluid [C]
% Tinitial = Inital temperature of solid/surface temperature [C]
% h   = heat transfer coefficient [W/(m2K)]
% m_f = mass flow in one fracture
% Cp_w = specific heat of water

As = 2*[0:0.1:L];

Tflowconst = Tinitial -(Tinitial - Tin)*exp(-h*As./(m_f*Cp_w));

end

%Analytic expression of a EGS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [TflowAN TmatrixAN] = EGSanalytic(k_r,rho_r,rho_w,Cp_r,u,H_f,Cp_w,Tin,Tinitial,X,Y,t,nx,t_vector);
%Analytic expression based on Sutter et al 2011
% Heat extraction from a EGS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Analytic equation
% Tm = Tw0 + (Tr0 - Tw0)*erf[ (x + beta*z)/(2*sqrt(alpha*t)) ]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

alpha = k_r(1,1)/(rho_r*Cp_r(1,1));
beta = k_r(1,1)/(rho_w*Cp_w*u*H_f/2);

%Expression for fluid temperatur, at time t
%Assumed fluid temperature = rock surface temperature
TflowAN = Tin + (Tinitial - Tin)*erf( beta*X(1,:)./(2*sqrt(alpha*t)) );

%Rock matrix temperature, at time t
TmatrixAN = Tin + (Tinitial - Tin)*erf( (Y + beta*X/(2*sqrt(alpha*t)) ));



end

%Temperature dependent Cp value for granite
function [Cp_r] = CP(T0)
%calculates Cp values for granite depending on temperature.

%Parameters
[yn xn] = size(T0);
Tn = ones(yn,xn)*20; %temperature of Cpn (20C)
Cpn = 800; %average value of measured Cp at 20 C frpm litterature

%Calculation
CpnT0 = 8.95*10^(-10)*T0.^3 - 2.13*10^(-6)*T0.^2 + 0.00172*T0 + 0.76;
CpnT = 8.95*10^(-10)*Tn.^3 - 2.13*10^(-6)*Tn.^2 + 0.00172*Tn + 0.76;

Cp_r = Cpn.*CpnT0./CpnT;
end

%Temperature dependent k value for granite
function [k_r] = K_r(T0)

%calculates k values for granite depending on temperature.

%Parameters
[yn xn] = size(T0);
Tn = ones(yn,xn)*25; %temperature of k (25C)
k25 = 3.5; %average value of measured k at 20 C from litterature

%Calculation
k0 = k25*(1.007 + 25*(0.0037 - 0.0074/k25));

k_r = k0./(1.007 + T0*(0.0036 - 0.0072/k0));


end